@@ -1,5 +1,5 @@
 /*******************************************************************
-   Copyright (C) 2001-7 Leo Breiman, Adele Cutler and Merck & Co., Inc.
+   Copyright (C) 2001-2012 Leo Breiman, Adele Cutler and Merck & Co., Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -14,6 +14,7 @@
 #include <R.h>
 #include "rf.h"
 
+
 void zeroInt(int *x, int length) {
     memset(x, 0, length * sizeof(int));
 }
@@ -27,7 +28,7 @@ void createClass(double *x, int realN, int totalN, int mdim) {
     int i, j, k;
     for (i = realN; i < totalN; ++i) {
         for (j = 0; j < mdim; ++j) {
-            k = (int) unif_rand() * realN;
+            k = (int) (unif_rand() * realN);
             x[j + i * mdim] = x[j + k * mdim];
         }
     }
@@ -148,7 +149,7 @@ void modA(int *a, int *nuse, const int nsample, const int mdim,
 }
 
 void Xtranslate(double *x, int mdim, int nrnodes, int nsample,
-		int *bestvar, int *bestsplit, int *bestsplitnext,
+		int *bestvar, double *bestsplit, double *bestsplitnext,
 		double *xbestsplit, int *nodestatus, int *cat, int treeSize) {
 /*
  this subroutine takes the splits on numerical variables and translates them
@@ -163,10 +164,10 @@ void Xtranslate(double *x, int mdim, int nrnodes, int nsample,
 	if (nodestatus[i] == 1) {
 	    m = bestvar[i] - 1;
 	    if (cat[m] == 1) {
-		xbestsplit[i] = 0.5 * (x[m + (bestsplit[i] - 1) * mdim] +
-				       x[m + (bestsplitnext[i] - 1) * mdim]);
+		    xbestsplit[i] = 0.5 * (x[m + ((int) bestsplit[i] - 1) * mdim] +
+				       x[m + ((int) bestsplitnext[i] - 1) * mdim]);
 	    } else {
-		xbestsplit[i] = (double) bestsplit[i];
+		    xbestsplit[i] = bestsplit[i];
 	    }
 	}
     }
@@ -229,7 +230,7 @@ void computeProximity(double *prox, int oobprox, int *node, int *inbag,
     for (i = 0; i < n; ++i) {
         for (j = i+1; j < n; ++j) {
             if (oobprox) {
-                if ((inbag[i] > 0) ^ (inbag[j] > 0)) {
+                if (! (inbag[i] > 0 || inbag[j] > 0) ) {
                     oobpair[j*n + i] ++;
                     oobpair[i*n + j] ++;
                     if (node[i] == node[j]) {
@@ -247,6 +248,23 @@ void computeProximity(double *prox, int oobprox, int *node, int *inbag,
     }
 }
 
+double pack(const int nBits, const int *bits) {
+    int i = nBits - 1;
+	double pack = bits[i];
+    for (i = nBits - 1; i > 0; --i) pack = 2.0 * pack + bits[i - 1];
+    return(pack);
+}
+
+void unpack(const double pack, const int nBits, int *bits) {
+    int i;
+    double x = pack;
+    for (i = 0; i <= nBits; ++i) {
+    	bits[i] = ((unsigned long) x & 1) ? 1 : 0;
+    	x = x / 2;
+    }
+}
+
+/*
 unsigned int pack(int nBits, int *bits) {
     int i = nBits;
 	unsigned int pack = 0;
@@ -255,44 +273,139 @@ unsigned int pack(int nBits, int *bits) {
 }
 
 void unpack(int nBits, unsigned int pack, int *bits) {
-/* pack is a 4-byte integer.  The sub. returns icat, an integer 
-   array of zeroes and ones corresponding to the coefficients 
-   in the binary expansion of pack. */
     int i;
     for (i = 0; i < nBits; pack >>= 1, ++i) bits[i] = pack & 1;
 }
+*/
 
-void F77_NAME(unpack)(int *nBits, unsigned int *pack, int *bits) {
-	unpack(*nBits, *pack, bits);
+void F77_NAME(unpack)(double *pack, int *nBits, int *bits) {
+	unpack(*pack, *nBits, bits);
 }
 
-#ifdef OLD
+void sampleDataRows(int populationSize, int sampleSize, int useWeights, 
+                    int withReplacement, double *weights, int *sampledIndices) {
+    /*At this point shouldn't need to check for sanity (sample > population)*/
+    /*Rprintf("Flags: replace %d useweights %d ", withReplacement, useWeights);*/
+    if(withReplacement) {
+        
+        if(useWeights) {
+            sampleWithReplacementWithWeights(sampleSize, populationSize, weights, sampledIndices);
+        }
+        else{
+            /*Rprintf("We will be sampling with replacement and no weights");*/
+            sampleWithReplacement(sampleSize, populationSize, sampledIndices);
+        }
+        
+    }
+    else{
+        if(useWeights) {
+            sampleWithoutReplacementWithWeights(sampleSize, populationSize, weights, sampledIndices);
+        }
+        else{
+            sampleWithoutReplacement(sampleSize, populationSize, sampledIndices);
+        }
+    }
+}
 
-double oldpack(int l, int *icat) {
-    /* icat is a binary integer with ones for categories going left
-     * and zeroes for those going right.  The sub returns npack- the integer */
+void sampleWithReplacementWithWeights(int sampleSize, int populationSize, double *weights, int *sampledIndices) {
+    int mflag = 0;
+    int numBoundaries = populationSize + 1;
+    double *boundaries = (double*)Calloc(numBoundaries, double);
+    calculateBoundaries(weights, boundaries, populationSize, numBoundaries);
+
+    double sample;
+    for(int i = 0; i < sampleSize; ++i) {
+        sample = unif_rand();
+        sampledIndices[i] = findInterval(boundaries, numBoundaries, sample, 1, 1,0, &mflag) - 1;
+    }
+}
+
+void sampleWithoutReplacementWithWeights(int sampleSize, int populationSize, double *weights, int *sampledIndices) {
+    /*A snippet to run once when rf is run to verify the output of find_interval, which is what we use to sample with weights*/
+    /*if(tree == 0){
+        int testflag = 0;
+        Rprintf("Testing the find interval function in r\n");
+        double testBoundaries[] = {0, .2, .6, .6, .6, .6, .8, .8, .8, 1};
+        int testInterval = findInterval(testBoundaries, 11, .7, 1, 1,0, &testflag);
+        Rprintf("The resultant test interval found was %d\n", testInterval);
+    }*/
+    int mflag = 0;
+    int numBoundaries = populationSize + 1;
+    double *weightsForTree = (double*)Calloc(populationSize, double);
+    
+    /*copy over weights to array so we don't mess with original weights when removing them*/
+    for(int i = 0; i < populationSize; i++){
+        weightsForTree[i] = weights[i];
+    }
+    double *boundaries = (double*)Calloc(numBoundaries, double);
+    int *populationTaken = (int*)Calloc(populationSize, int);
+    zeroInt(populationTaken, populationSize);
+    calculateBoundaries(weightsForTree, boundaries, populationSize, numBoundaries);
+    int first;
+    int sampledIndex;
+    int sampledIndexCopy;
+    double sample;
+    
+    for(int i = 0; i < sampleSize; ++i) {
+        sample = unif_rand();
+        sampledIndex = findInterval(boundaries, numBoundaries, sample, 1, 1,0, &mflag) - 1;
+        first = sampledIndex;
+        sampledIndexCopy = sampledIndex;
+        sampledIndices[i] = sampledIndex;
+        populationTaken[sampledIndex] = -1;
+        removeWeightAndNormalize(weightsForTree, sampledIndex, populationSize);
+        calculateBoundaries(weightsForTree, boundaries, populationSize, numBoundaries);
+    }
+}
+
+void sampleWithReplacement(int sampleSize, int populationSize, int *sampledIndices) {
+    for(int i = 0; i < sampleSize; ++i) {
+        sampledIndices[i] = unif_rand() * populationSize;
+    }
+}
+
+void sampleWithoutReplacement(int sampleSize, int populationSize, int *sampledIndices) {
+    int *indices = (int*)Calloc(populationSize, int);
+    for(int i = 0; i < populationSize; ++i) {
+        indices[i] = i;
+    }
+    int indexOfLast = populationSize - 1;
     int k;
-    double pack = 0.0;
-
-    for (k = 0; k < l; ++k) {
-	if (icat[k]) pack += R_pow_di(2.0, k);
-    }
-    return(pack);
-}
-
-
-void oldunpack(int l, int npack, int *icat) {
-/*
- * npack is a long integer.  The sub. returns icat, an integer of zeroes and
- * ones corresponding to the coefficients in the binary expansion of npack.
- */
-    int i;
-    zeroInt(icat, 32);
-    icat[0] = npack % 2;
-    for (i = 1; i < l; ++i) {
-	npack = (npack - icat[i-1]) / 2;
-	icat[i] = npack % 2;
+    for(int i = 0; i < sampleSize; i++) {
+        int currentSample = unif_rand() * (indexOfLast + 1);
+        k = indices[currentSample];
+        swapInt(indices[currentSample], indices[indexOfLast]);
+        indexOfLast--;
+        sampledIndices[i] = k;
     }
 }
 
-#endif /* OLD */
+void removeWeightAndNormalize(double *weights, int indexToRemove, int populationSize) {
+    double weightToRemove = weights[indexToRemove];
+    double sumOfWeights = 1 - weightToRemove;
+    weights[indexToRemove] = 0;
+    
+    for(int i = 0; i < populationSize; i++) {
+        weights[i] /= sumOfWeights;
+    }
+}
+
+void normalizeWeights(double *weights, int numWeights){
+    double weightSum = 0;
+    for(int i = 0; i < numWeights; i++){
+        weightSum += weights[i];
+    }
+    for(int i = 0; i < numWeights; i++){
+        weights[i] /= weightSum;
+    }
+}
+
+void calculateBoundaries(double *weights, double *boundaries, int populationSize, int numBoundaries) {
+    zeroDouble(boundaries, numBoundaries);
+    double currentSum = 0;
+    boundaries[0] = currentSum;
+    for(int i = 1; i <= populationSize; i++) {
+        currentSum += weights[i - 1];
+        boundaries[i] = currentSum;
+    }
+}
